unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, MMSystem, ExtCtrls, ComCtrls;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    FIsOpened: boolean;
    FDeviceID: Word;
    FDeviceName:string;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}
const
  FFourCC = 'IOCB'; // acronym for IO CallBack (but it really does not matter - choose as you wish)

var
  FOpened: boolean = False;
  f:TFileStream;
  filename:string = 'test.avi';

function MyIOCallback(lpmmioinfo: PChar; uMessage: UINT; lParam1, lParam2: LPARAM): Longint stdcall;
// This function is called and will provide access to the file. The MM system "thinks"
// this is a file, while we use the global memory stream.
var
  AInfo: pmmioInfo; c:integer;
begin
  AInfo := pmmioInfo(lpmmioinfo);
  case uMessage of
  MMIOM_OPEN:
    // Open the "file"
    begin
      if FOpened then
      begin
        Result := 0;
        exit;
      end;
      FOpened := True;
      f:=TFileStream.Create(filename,fmOpenRead or fmShareDenyNone);
//      FileMode:=fmOpenRead;// reading from a readonly running exe
      AInfo.lDiskOffset := 0;
//      f.seek(startoffset,soFromBeginning);// reading from an exe, where the movie is at the end. need to know the startoffset
      Result := 0;
    end;
  MMIOM_CLOSE:
      // Close the "file"
    begin
      f.free;
      FOpened:=false;
      Result := 0;
    end;
  MMIOM_READ:
    // Read data from the "file"
    begin
      // Copy from memory - no checking
{      c:=f.read(pointer(lParam1)^,lParam2);
      AInfo.lDiskOffset := AInfo.lDiskOffset + lParam2;
      Result := lParam2;{}
      c:=f.read(pointer(lParam1)^,lParam2);
      AInfo.lDiskOffset := AInfo.lDiskOffset + c;
      Result := c;
    end;
  MMIOM_SEEK:
    // Seek new position in the "file"
    begin
      case lParam2 of
      // From beginning:
      0: AInfo.lDiskOffset := lParam1;
      // From current:
      1: AInfo.lDiskOffset := AInfo.lDiskOffset + lParam1;
      // From end:
      2: AInfo.lDiskOffset := f.size - 1 - lParam1;
      end;
//      f.seek(startoffset+AInfo.lDiskOffset,soFromBeginning);// see above about the startoffset
      f.seek(AInfo.lDiskOffset,soFromBeginning);
      Result := AInfo.lDiskOffset;
    end;
  else
    // Unexpected msgs. For instance, we do not process MMIOM_WRITE in this sample
    Result := -1;
  end;
end;

function findStr(subStr,str:string; start:integer):integer;
var s:string;
begin
  s:=copy(str,start,length(str));
  result:=start+pos(subStr,s)-1;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  OpenParm: TMCI_Open_Parms;
  GenParm: TMCI_Generic_Parms;
  FFlags: Longint;
  FError: Longint;
  ErrMsg: array[0..4095] of Char;
  AError: string;

  PlayParm: TMCI_Play_Parms;
  AWindowParm: TMCI_Anim_Window_Parms;
  StatusParms: TMCI_STATUS_PARMS;
  SetParams: TMCI_Set_Parms;

  size:PAnsiChar;
  s1,s2:integer;
  w,h:integer;
begin
  if not assigned(mmioInstallIOProc(mmioStringToFourCC(FFourCC, 0), @MyIOCallback, MMIO_INSTALLPROC or MMIO_GLOBALPROC)) then
  begin
    ShowMessage('Could not install IO proc');
    exit;
  end;

  // Open the file
  if FIsOpened then
  begin
    GenParm.dwCallback := Handle;
    mciSendCommand(FDeviceID, mci_Close, 0, Longint(@GenParm));
    FIsOpened := False;
  end;

  FillChar(OpenParm, SizeOf(TMCI_Open_Parms), 0);

  // Open a specific device type - obviously, and unfortunately,
  // the memory play function does NOT work for mpegvideo.
//        OpenParm.lpstrDeviceType := 'MPEGVideo'; // does not work
  OpenParm.lpstrDeviceType := 'avivideo';

  // Set the name of the file with extension .IOCB+ to invoke our own
  // I/O callbacks. This way the file is read from memory
  OpenParm.lpstrElementName := 'test.IOCB+';
  // OpenParm.lpstrElementName := PChar(edFilename.Text); // alternative, directly read file
  OpenParm.dwCallback := Handle;
  OpenParm.lpstrAlias:=PAnsiChar('movie'+inttostr(random(100)));

  FFlags := mci_Open_Type or MCI_OPEN_ELEMENT or MCI_OPEN_ALIAS;
  FError := mciSendCommand(0, mci_Open, FFlags, Longint(@OpenParm));

  if FError <> 0 then
  begin
    // problem opening device
    if not mciGetErrorString(FError, ErrMsg, SizeOf(ErrMsg)) then
      AError := 'Unknown error'
    else
      SetString(AError, ErrMsg, StrLen(ErrMsg));
    showmessage('Error: '+ErrMsg);
  end else
  begin
    FIsOpened := True;
    FDeviceID := OpenParm.wDeviceID;
    FDeviceName:= OpenParm.lpstrAlias;

    getmem(size,129);
//        mciSendCommand(FDeviceID,mci_
    if mciSendString(PAnsiChar('Where ' + OpenParm.lpstrAlias + ' destination'), size, 128, 0)=0 then
    begin
      s1:=findStr(' ',size,1);
      s2:=findStr(' ',size,s1 + 1);
      s1:=findStr(' ',size,s2 + 1);
      w:=strtoint(copy(size, s2, s1 - s2));
      h:=strtoint(copy(size, s1 + 1,length(size)));
      if (w>0) and (h>0) then
      begin
        ClientWidth:=w;
        ClientHeight:=h;
        left:=0;// video window position
        top:=0;
        Caption:='Video Window Caption';
        show;
      end;
      freemem(size);
    end;
//   start playing
    if not FIsOpened then exit;
    AWindowParm.Wnd := Longint(Panel1.Handle);
//      FError :=
    mciSendCommand(FDeviceID, mci_Window, mci_Wait or mci_Anim_Window_hWnd, Longint(@AWindowParm) );

    SetParams.dwTimeFormat:=MCI_FORMAT_MILLISECONDS;
    mciSendCommand(FDeviceID,MCI_SET,MCI_SET_TIME_FORMAT,Longint(@SetParams));

    StatusParms.dwItem:=MCI_STATUS_LENGTH;
    StatusParms.dwTrack:=0;
    mciSendCommand(FDeviceID,MCI_STATUS,MCI_STATUS_ITEM,longint(@StatusParms));
//    total:=StatusParms.dwReturn;
//    TrackBar.Max:=total div 1000;
//    debug('MCI - total length (in seconds) = '+inttostr(TrackBar.Max));

//    totalLength:=total div 1000;

      // play
    PlayParm.dwCallback := Handle;
//      FError :=
    mciSendCommand(FDeviceID, mci_Play, 0, Longint(@PlayParm));
  end;
end;

procedure TForm1.FormDestroy(Sender: TObject);
var
  GenParm: TMCI_Generic_Parms;
begin
  if FIsOpened then
  begin
    GenParm.dwCallback := Handle;
    mciSendCommand(FDeviceID, mci_Close, 0, Longint(@GenParm));
    FIsOpened := False;
    // remove mmio callback
    mmioInstallIOProc(mmioStringToFourCC(FFourCC, 0), nil, MMIO_REMOVEPROC);
  end;
end;

end.
